from itertools import combinations

import pandas as pd


def graph_formatter(
    df: pd.DataFrame,
    col: str,
):
    if col not in df.columns:
        raise ValueError(
            f"Coluna '{col}' não encontrada.\nColunas disponíveis: {list(df.columns)}"
        )

    df[col] = df[col].fillna("").astype(str)

    edge_weights = {}
    nodes_set = set()

    for _, row in df.iterrows():
        nodes_data = [a.strip() for a in row[col].split(";") if a.strip()]

        for data in nodes_data:
            nodes_set.add(data)

        for node1, node2 in combinations(nodes_data, 2):
            edge_key = tuple(sorted((node1, node2)))
            edge_weights[edge_key] = edge_weights.get(edge_key, 0) + 1

    nodes = []
    edges = []

    nodes = [{"data": {"id": node, "label": node}} for node in nodes_set]

    for (source, target), weight in edge_weights.items():
        edges.append(
            {
                "data": {
                    "source": source,
                    "target": target,
                    "weight": weight,
                }
            }
        )

    return {"nodes": nodes, "edges": edges}



import pandas as pd


def get_counts(df: pd.DataFrame, column_name: str, output_key_name: str):
    if column_name not in df.columns:
        return []

    series = df[column_name].fillna("").astype(str)

    counts = series.str.split(";").explode().str.strip().value_counts()

    result_data = []
    for term, count in counts.items():
        if term and str(term).lower() not in ["nan", "n/a", ""]:
            result_data.append({output_key_name: term, "count": int(count)})
    return result_data


def count_data(df: pd.DataFrame, col: str):
    chart_data = get_counts(df, col, "label")

    charts_data = {"data": chart_data}

    return charts_data



import pandas as pd
from unidecode import unidecode


def keep_columns(
    df: pd.DataFrame, columns_to_keep: list[tuple[str, int]]
) -> pd.DataFrame:
    existing_columns = [col for col, _ in columns_to_keep if col in df.columns]
    df = df[existing_columns].fillna("N/A")

    return df


def process_wos_data(df: pd.DataFrame, header: list[tuple[str, int]]) -> pd.DataFrame:
    for column, new_index in header:
        replaced_column_data = df[column].fillna("N/A")
        df.pop(column)
        df.insert(new_index, column, replaced_column_data)

    df.loc[:, "AU"] = df["AU"].apply(
        lambda x: (
            "; ".join(
                [
                    " ".join(
                        [
                            f"{word[0]}.{word[1:]}"
                            if len(word) == 2 and word.isupper()
                            else word
                            for word in name.split()
                        ]
                    )
                    + "."
                    if "," in name
                    else name
                    for name in x.split(";")
                ]
            )
            if pd.notna(x)
            else x
        )
    )

    df.loc[:, "AU"] = df["AU"].apply(lambda x: x.replace(",", ""))

    return df


def process_scopus_data(
    df: pd.DataFrame, header: list[tuple[str, int]]
) -> pd.DataFrame:
    for column, new_index in header:
        replaced_column_data = df[column].fillna("N/A")
        df.pop(column)
        df.insert(new_index, column, replaced_column_data)

    for column in df.columns:
        if df[column].dtype == "object":
            if column == "Abstract":
                df.loc[:, column] = df[column].apply(
                    lambda x: (
                        x.replace("[No abstract available]", "N/A")
                        if pd.notna(x)
                        else x
                    )
                )

                df.loc[:, column] = df[column].apply(
                    lambda x: (
                        unidecode(x).replace('"', "").lower().strip()
                        if pd.notna(x)
                        else x
                    )
                )

    return df


def merge_and_process(
    df_target: pd.DataFrame, df_visitor: pd.DataFrame, mapping: dict, subset_cols: list
) -> pd.DataFrame:
    visitor_standardized = df_visitor.rename(columns=mapping)
    df = pd.concat([df_target, visitor_standardized], ignore_index=True)
    doi_col = next((c for c in ["DOI", "DI"] if c in df.columns), None)

    if doi_col:
        mask_duplicates = df.duplicated(subset=doi_col)
        mask_na_values = df[doi_col].isna()
        mask_to_keep = ~mask_duplicates | mask_na_values
        df = df[mask_to_keep]

    df = df.drop_duplicates(subset=subset_cols, keep="first")

    return df
